@@ -19,7 +19,7 @@
 {
     #####################################################################################################################################
     # Validation de l'argument fourni en entre
-    if (any((vt %% 1)!=0)) stop("the 'vt' components must be integers")
+    if (any((vt %% 1)!=0) || any(vt<=0)) stop("the 'vt' components must be positive integers")
     #####################################################################################################################################
 
     I<-length(vt)
